package jfml.interoperatibility.embeddedsystem.arduino.test;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import jfml.FuzzyInferenceSystem;
import jfml.JFML;
import jfml.interoperatibility.embeddedcontroller.EmbeddedController;
import jfml.interoperatibility.embeddedcontroller.system.EmbeddedControllerSystem;
import jfml.interoperatibility.embeddedsystem.EmbeddedSystem;
import jfml.interoperatibility.embeddedsystem.EmbeddedVariable;
import jfml.interoperatibility.embeddedsystem.arduino.ArduinoPin;
import jfml.interoperatibility.embeddedsystem.arduino.EmbeddedSystemArduinoUsb;
import jfml.interoperatibility.embeddedsystem.arduino.EmbeddedVariableArduino;
import jfml.interoperatibility.embeddedsystem.connection.ArduinoExceptionSerial;
import jfml.interoperatibility.embeddedsystem.sensor.AggregatedSensor;
import jfml.interoperatibility.embeddedsystem.sensor.Sensor;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoAggregatedSensorAV;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoAggregatedSensorDQ;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoAggregatedSensorO;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoAggregatedSensorRD;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoAggregatedSensorV;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoHC_SR04;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoH_BRIDGE_L298N;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoMPU6050;
import jfml.interoperatibility.embeddedsystem.sensor.arduino.ArduinoSERVO;
import jfml.knowledgebase.variable.KnowledgeBaseVariable;

public class ExampleArduinoRobotUSB {

	public static void main(String[] args) throws InterruptedException, ArduinoExceptionSerial, IOException {
		File fml = new File("Examples/XMLFiles/RobotIEEEstd1855.xml");
		String port = "YOUR_USB_PORT";
		int time = 1000;
		int baud = 2000000;
		String csvFile = "./Examples/Arduino/"+ fml.getName()+".csv";
		boolean debug = true;
		boolean csv = false;

		if(args.length > 0) {
            fml = new File(args[0]);
            port = new String(args[1]);
            baud = Integer.parseInt(args[2]);
            time = Integer.parseInt(args[3]);
            debug = Boolean.parseBoolean(args[4]);
            csv = Boolean.parseBoolean(args[5]);
            csvFile = new String(args[6]);
		}
		//CREATE A CONTROLLER
		EmbeddedController controller = new EmbeddedControllerSystem();
		
		//Control the keys Ctrl+C
        Runtime.getRuntime().addShutdownHook(new Thread(){
            @Override
            public void run(){
                System.out.println("!!!Ctrl+C pressed!!!");
                controller.closeConnection();
            }
        });
		
		//loading Fuzzy System from an XML file according the standard IEEE 1855
		FuzzyInferenceSystem fs = JFML.load(fml);
		
		// get variables
		KnowledgeBaseVariable rd =  fs.getVariable("rd");
		KnowledgeBaseVariable dq =  fs.getVariable("dq");
		KnowledgeBaseVariable o =  fs.getVariable("o");
		KnowledgeBaseVariable v =  fs.getVariable("v");
		KnowledgeBaseVariable av =  fs.getVariable("av");
		KnowledgeBaseVariable la =  fs.getVariable("la");

		/* MAPPING VARIABLES WITH SENSORS (True means is Aggregated sensor) */
		// min_distance=10, max_distance=200, thresold=3, dwall=50, buffer, sizeBuffer, multilib, Aggregated
		Sensor rd_front = new ArduinoHC_SR04(rd.getName()+"front", ArduinoPin.PIN_40, ArduinoPin.PIN_41, 10, 200, true, 3, true, true);
		//buffer, sizeBuffer, multilib, Aggregated
		Sensor rd_front2 = new ArduinoHC_SR04(rd.getName()+"front2", ArduinoPin.PIN_48, ArduinoPin.PIN_49, true, 3, false, true);
		Sensor rd_right = new ArduinoHC_SR04(rd.getName()+"right", ArduinoPin.PIN_50, ArduinoPin.PIN_51, true, 3, false, true);
		Sensor rd_right2 = new ArduinoHC_SR04(rd.getName()+"right2", ArduinoPin.PIN_32, ArduinoPin.PIN_33, true, 3, false, true);
		Sensor dq_left = new ArduinoHC_SR04(dq.getName()+"left", ArduinoPin.PIN_30, ArduinoPin.PIN_31, true, 3, false, true);
		//minfs=-1, maxfs=1, min_angle=-45, max_angle=135, factor=45
		Sensor angular_velocity = new ArduinoSERVO(av.getName(), ArduinoPin.PIN_9, -1, 1, 45, 135, 45, true);
		//minfs=-1, maxfs= 1, minSpeed=40, maxSpeed=70, factor=15
		Sensor linear_acceleration = new ArduinoH_BRIDGE_L298N(la.getName(), ArduinoPin.PIN_6, ArduinoPin.PIN_7, ArduinoPin.PIN_4, ArduinoPin.PIN_2, ArduinoPin.PIN_1, ArduinoPin.PIN_5, -1, 1, 40, 70, 15);
		//min=-45, max=45, Aggregated
		Sensor orientation = new ArduinoMPU6050(o.getName(), -45, 45, true);
		
		/* Aggregated SENSOR RD */
		ArrayList<Sensor> sensorsRD = new ArrayList<>();
		sensorsRD.add(rd_front);
		sensorsRD.add(rd_front2);
		sensorsRD.add(rd_right);
		sensorsRD.add(rd_right2);
		//minfs=0, maxfs=3, threshold=6, dwall=50, buffer=true
		AggregatedSensor rdSensor = new ArduinoAggregatedSensorRD(rd.getName(), sensorsRD, 0, 3, 6, 50, true);
		
		/* Aggregated SENSOR DQ */
		ArrayList<Sensor> sensorsDQ = new ArrayList<>();
		sensorsDQ.add(dq_left);
		AggregatedSensor dqSensor = new ArduinoAggregatedSensorDQ(dq.getName(), sensorsDQ, 0, 2);
		
		/* Aggregated SENSOR O */
		ArrayList<Sensor> sensorsO = new ArrayList<>();
		sensorsO.add(rd_right);
		sensorsO.add(rd_right2);
		AggregatedSensor OSensor = new ArduinoAggregatedSensorO(o.getName(), sensorsO, -45, 45);
		
		/* Aggregated SENSOR AV */
		ArrayList<Sensor> sensorsAV = new ArrayList<>();
		sensorsAV.add(angular_velocity);
		AggregatedSensor avSensor = new ArduinoAggregatedSensorAV(av.getName(), sensorsAV, -1, 1, 18);

		/* Aggregated SENSOR V */
		AggregatedSensor vSensor = new ArduinoAggregatedSensorV(v.getName(), -1, 1, 50);

		/* EMBEDDED VARIABLES */
		ArrayList<EmbeddedVariable> embeddedVariables = new ArrayList<>();
		embeddedVariables.add(new EmbeddedVariableArduino(0, rd, rd_front));
		embeddedVariables.add(new EmbeddedVariableArduino(0, rd, rd_front2));
		embeddedVariables.add(new EmbeddedVariableArduino(0, rd, rd_right));
		embeddedVariables.add(new EmbeddedVariableArduino(0, rd, rd_right2));
		embeddedVariables.add(new EmbeddedVariableArduino(0, rd, rdSensor));
		embeddedVariables.add(new EmbeddedVariableArduino(1, dq, dq_left));
		embeddedVariables.add(new EmbeddedVariableArduino(1, dq, dqSensor));
		embeddedVariables.add(new EmbeddedVariableArduino(2, o, orientation));
		embeddedVariables.add(new EmbeddedVariableArduino(2, o, OSensor));
		embeddedVariables.add(new EmbeddedVariableArduino(3, v, vSensor));
		embeddedVariables.add(new EmbeddedVariableArduino(4, la, linear_acceleration));
		embeddedVariables.add(new EmbeddedVariableArduino(5, av, angular_velocity));
		embeddedVariables.add(new EmbeddedVariableArduino(5, av, avSensor));
		
		//CREATE A EMBEDDED SYSTEM ARDUINO USB
		EmbeddedSystem arduino = new EmbeddedSystemArduinoUsb("Arduino-Robot", port, baud,  embeddedVariables, 10);
		arduino.getConnection().setDebug(debug);
		
		//CREATE .INO FILE
		arduino.createRunnableEmbeddedFile("./Examples/Arduino/"+fml.getName());
		
		//CREATE ARDUINOS LIST
		ArrayList<EmbeddedSystem> arduinos = new ArrayList<>();
		arduinos.add(arduino);
		
		//SET EMBEDDED SYSTEMS TO CONTROLLER
		controller.setEmbeddedSystems(arduinos);

		//SET CONTROLLER FS TO CONTROLLER
		controller.setFs(fs);
		
		//CONTROL THE ARGUMENTS
		if(debug && csv) {
			controller.setDebugModeCSV(csv);
			controller.setDebugMode(debug);
			controller.run(time);
		}
		else if (csv) {
			controller.setDebugModeCSV(csv);
			controller.setCsvFile(csvFile);
			controller.run(time);	         
		}
		else if (debug) {
			controller.setDebugMode(debug);
			controller.run(time);
		}
		else {
			controller.run(time);
		}
		
	}
}
